﻿/*
 * File Name      : EciB3.cs
 *
 * Product        : CICS Transaction Gateway
 *
 * Description    : This sample shows the basic use of the CICS Transaction
 *                  Gateway ECI API. It queries the Gateway for a list of
 *                  available servers and then launches a transaction on the
 *                  server chosen.
 *
 * Pre-Requisites : This sample requires the CICS Program EC03 to be installed
 *                  on your CICS server for operation. An IPIC connection to
 *                  CICS required to flow channel and container data.
 *
 *                  This sample must be compiled to target the x86 processor
 *                  architecture, rather than the default of Any CPU. If the
 *                  sample is compiled for Any CPU, it may fail to execute on
 *                  64 bit platforms.
 *
 *                                       
 * Licensed Materials - Property of IBM
 * 
 * 5724-I81 5655-W10
 * 
 * (c) Copyright IBM Corp. 2010
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * 
 * Status: Version 1 Release 0
 *                                     
 *
 *         The following code is sample code created by IBM Corporation. This
 *         sample code is not part of any standard IBM product and is provided
 *         to you solely for the purpose of assisting you in the development of
 *         your applications.  The code is provided 'AS IS', without warranty
 *         or condition of any kind.  IBM shall not be liable for any damages
 *         arising out of your use of the sample code, even if IBM has been
 *         advised of the possibility of such damages.
 */

using System;
using System.Collections.Generic;
using System.Text;

namespace IBM.CTG.Samples
{
    class EciB3
    {

        public const string copyright =
        //                                                        
        "\n" +
        "Licensed Materials - Property of IBM " +
        "5724-I81 5725-B65 5655-W10" +
        "\n" +
        "(c) Copyright IBM Corp. 2010  All Rights Reserved. " +
        "US Government Users Restricted Rights - Use, duplication or " +
        "disclosure restricted by GSA ADP Schedule Contract with " +
        "IBM Corp." +
        "\n";
        //                                                      


        //Name of the CICS program to execute
        private const string ProgramName = "EC03";

        //Maximum number of flow attempts
        private const int MaxFlowAttempts = 3;

        //Name of Channel to pass to EC03 program
        private const string CHANNEL_NAME = "SAMPLECHANNEL";

        //Name of Container to pass to EC03 program
        private const string CONTAINER_NAME = "INPUTDATA";

        //Objects used by the class
        private string gwyHostName = "localhost";
        private int gwyPort = 2006;

        static void Main(string[] args)
        {
            EciB3 eciApp = new EciB3();
            //Display program banner
            Console.WriteLine("CICS Transaction Gateway Basic ECI Sample 3");
            Console.WriteLine();

            //Display usage information
            Console.WriteLine("Usage: EciB3.exe [Gateway host name] [Gateway port number]");
            Console.WriteLine();

            if (eciApp.ParseArguments(args)){
                try{
                    eciApp.Start();

                }catch (Exception e){
                    //Output exception details
                    Console.WriteLine(e.Message);
                    Console.WriteLine(e.StackTrace);
                }
            }
        }

        /// <summary>
        /// Parses the command line arguments specified when the application was launched.
        /// </summary>
        /// <param name="args">An array of command line arguments.</param>
        /// <returns><see langword="true"/> if the arguments are parsed successfully; otherwise <see langword="false"/>.</returns>
        public bool ParseArguments(string[] args)
        {

            //Check argument count
            if (args.Length > 2){
                for (int i = 2; i < args.Length; i++){
                    Console.WriteLine("ERROR: Unexpected argument: {0}", args[i]);
                }
                return false;
            }

            //Parse Gateway port number
            if (args.Length > 1){
                if (!int.TryParse(args[1], out gwyPort)){
                    Console.WriteLine("ERROR: Invalid port number specified");
                    return false;
                }
            }

            //Parse Gateway host name
            if (args.Length > 0){
                gwyHostName = args[0];
            }

            Console.WriteLine("The address of the Gateway has been set to {0} port {1}", gwyHostName, gwyPort);
            Console.WriteLine();

            return true;
        }

        /// <summary>
        /// Connects to the CICS Transaction Gateway and flows an ECI request.
        /// </summary>
        /// <remarks>
        /// The user is prompted to select a CICS server to execute the EC03 program on.
        /// The contents of each Container in the returned Channel is displayed.
        /// </remarks>
        public void Start()
        {

            string cicsServer;
            EciRequest eciReq;

            //Open the Gateway connection with an infinite timeout period
            //The 'using' C# construct ensures the GatewayConnection is closed
            //when it goes out of scope, even if an exception is thrown

            //A System.Net.SocketException will be thrown if the connection
            //attempt fails, and an IBM.CTG.GatewayException will be thrown if
            //an error occurs in the Gateway daemon
            using (GatewayConnection gwyConnection = new GatewayConnection(gwyHostName, gwyPort)){

                //Connected to the Gateway daemon

                //Get name of CICS server from the user
                cicsServer = SelectServer(gwyConnection);
                if (cicsServer == null){
                    //No servers defined, or user chose to quit
                    return;
                }

                //Prepare the EciRequest
                eciReq = new EciRequest();
                eciReq.ServerName = cicsServer;
                eciReq.Program = ProgramName;
                eciReq.ExtendMode = EciExtendMode.EciNoExtend;

                //The 'using' C# construct ensures the Channel is
                //correctly disposed of when it goes out of scope, even if
                //an exception is thrown.
                using (Channel channel = CreateChannel()){
                    //Set the Channel on the ECIRequest
                    eciReq.Channel = channel;

                    //Flow the request
                    if (FlowRequest(gwyConnection, eciReq)){

                        //Display returned Channel
                        DisplayChannelContents(eciReq.Channel);

                    }

                }
            }
        }

        /// <summary>
        /// Creates a Channel with one CHAR Container to pass to the EC03 program
        /// </summary>
        /// <returns>A Channel object to be used for the ECI request</returns>
        private Channel CreateChannel()
        {
            Console.WriteLine("Enter text to send to the CICS program (max 128 chars):");
            String text = Console.ReadLine();

            Channel channel = new Channel(CHANNEL_NAME);
            channel.CreateContainer(CONTAINER_NAME, text);

            return channel;
        }

        /// <summary>
        /// Prints a list of Containers in the Channel and the contents of each Container
        /// </summary>
        private void DisplayChannelContents(Channel channel)
        {
            Console.WriteLine("Program EC03 returned containers:");
            Container[] containers = channel.GetContainers();

            foreach (Container aContainer in containers){
                if (aContainer.Type == ContainerType.BIT){
                    Console.Write("    [BIT] {0,-16} = ", aContainer.Name);
                    Console.WriteLine(BitConverter.ToString(aContainer.GetBitData()));
                }else{
                    Console.Write("   [CHAR] {0,-16} = ", aContainer.Name);
                    Console.WriteLine(aContainer.GetCharData());
                }
            }

        }

        /// <summary>
        /// Flows the ECI request, up to a maximum of MaxFlowAttempts attempts.
        /// </summary>
        /// <returns><see langword="true"/> if the flow succeeds; <see langword="false"/> if MaxFlowAttempts is reached.</returns>
        /// <remarks>
        /// If the flow returns a security error, the user is prompted to enter CICS logon credentials.
        /// </remarks>
        private bool FlowRequest(GatewayConnection gwyConnection, EciRequest eciReq)
        {

            int attempts = 0;

            while (attempts < MaxFlowAttempts){

                //Flow throws a System.Net.SocketException if a connection
                //failure occurs, or an IBM.CTG.GatewayException if an error
                //occurs in the Gateway daemon
                gwyConnection.Flow(eciReq);

                //Test return code
                switch (eciReq.EciReturnCode){
                    case EciReturnCode.EciNoError:
                        //Flow succeeded
                        return true;

                    case EciReturnCode.EciErrSecurityError:
                        //Security error, prompt for CICS logon credentials
                        Console.Write("Enter CICS user id : ");
                        eciReq.UserId = Console.ReadLine();
                        Console.Write("Enter CICS password: ");
                        eciReq.Password = Console.ReadLine();
                        break;

                    case EciReturnCode.EciErrTransactionAbend:
                        //Transaction abended
                        Console.WriteLine("The transaction abended: {0}", eciReq.AbendCode);
                        return false;

                    case EciReturnCode.EciErrNoCics:
                        //CICS server not found
                        Console.WriteLine("CICS server not found");
                        return false;

                    default:
                        //Other error return code
                        Console.WriteLine("ECI return code: {0} ({1})", eciReq.EciReturnCode.ToString(), (int)eciReq.EciReturnCode);
                        return false;

                }

                //Increment number of attempts
                attempts++;
            }

            //Number of attempts exceeded MaxFlowAttempts
            return false;
        }


        /// <summary>
        /// Prompts the user to select a server to execute the EC03 program on.
        /// </summary>
        /// <returns>The name of the CICS server, as defined in the CICS Transaction Gateway.</returns>
        private string SelectServer(GatewayConnection gwyConnection)
        {

            IList<CicsServer> servers;
            string chosenServer = null;

            //Retrieve list of CICS servers from the Gateway

            //ListSystems throws a System.Net.SocketException if a connection
            //failure occurs, or an IBM.CTG.GatewayException if an error occurs
            //in the Gateway daemon
            servers = gwyConnection.ListSystems();

            if (servers.Count == 0){
                //There are no servers defined in the Gateway
                Console.WriteLine("No CICS servers have been defined in the Gateway");
                return null;
            }

            while (chosenServer == null){

                //Display a list of available CICS servers
                Console.WriteLine("CICS servers defined in the Gateway:");

                for (int i = 0; i < servers.Count; i++){
                    Console.WriteLine("   {0,2}. {1,-8} - {2}", (i + 1), servers[i].Name, servers[i].Description);
                }
                Console.WriteLine();

                //Prompt for user selection
                Console.Write("Choose server to connect to, or Q to quit: ");
                chosenServer = Console.ReadLine();

                //Parse user selection
                if (chosenServer.Equals("Q", StringComparison.InvariantCultureIgnoreCase)){
                    //User chose to quit
                    return null;
                }

                int serverNum;
                if (int.TryParse(chosenServer, out serverNum)){

                    //Validate number
                    if ((serverNum < 1) || (serverNum > servers.Count)){
                        Console.WriteLine("Invalid selection");
                        chosenServer = null;
                    }else{
                        chosenServer = servers[serverNum - 1].Name;
                    }

                }else{
                    //Input was not a number
                    Console.WriteLine("Invalid selection");
                    chosenServer = null;
                }

                Console.WriteLine();
            }

            return chosenServer;
        }
    }
}
