﻿
' File Name      : EciB3.vb
'
' Product        : CICS Transaction Gateway
'
' Description    : This sample shows the basic use of the CICS Transaction
'                  Gateway ECI API. It queries the Gateway for a list of
'                  available servers and then launches a transaction on the
'                  server chosen.
'
' Pre-Requisites : This sample requires the CICS Program EC03 to be installed
'                  on your CICS server for operation.  An IPIC connection to
'                  CICS required to flow channel and container data.
'
'                  This sample must be compiled to target the x86 processor
'                  architecture, rather than the default of Any CPU. If the
'                  sample is compiled for Any CPU, it may fail to execute on
'                  64 bit platforms.
'
'                                       
' Licensed Materials - Property of IBM
' 
' 5724-I81 5655-W10
' 
' (c) Copyright IBM Corp. 2008, 2010
' 
' US Government Users Restricted Rights - Use, duplication or
' disclosure restricted by GSA ADP Schedule Contract with
' IBM Corp.
' 
' Status: Version 1 Release 0
'                                     
'
'         The following code is sample code created by IBM Corporation. This
'         sample code is not part of any standard IBM product and is provided
'         to you solely for the purpose of assisting you in the development of
'         your applications.  The code is provided 'AS IS', without warranty
'         or condition of any kind.  IBM shall not be liable for any damages
'         arising out of your use of the sample code, even if IBM has been
'         advised of the possibility of such damages.

'Enforce strong type checking
Option Strict On
'Require all variables to be declared
Option Explicit On

Imports System
Imports System.Text
Imports System.Collections.Generic

'Namespace IBM.CTG contains all the CICS Transaction Gateway types
Imports IBM.CTG

Namespace IBM.CTG.Samples

    Public Class EciB3

        Public Const copyright As String = _
        "\n" + _
        "Licensed Materials - Property of IBM " + _
        "5724-I81 5725-B65 5655-W10 " + _
        "\n" + _
        "(c) Copyright IBM Corp. 2010  All Rights Reserved. " + _
        "US Government Users Restricted Rights - Use, duplication or " + _
        "disclosure restricted by GSA ADP Schedule Contract with " + _
        "IBM Corp." + _
        "\n"

        'Name of the CICS program to execute
        Private Const ProgramName As String = "EC03"

        'Maximum number of flow attempts
        Private Const MaxFlowAttempts As Integer = 3

        'Name of channel to pass to EC03 program
        Private Const CHANNEL_NAME As String = "SAMPLECHANNEL"

        'Name of container to pass to EC03 program
        Private Const CONTAINER_NAME As String = "INPUTDATA"

        'Objects used by the class
        Private gwyHostName As String = "localhost"
        Private gwyPort As Integer = 2006

        ''' <summary>
        ''' Application entry point.
        ''' </summary>
        ''' <param name="args">Command line arguments passed to the application.</param>
        Public Shared Sub Main(ByVal args As String())
            Dim eciApp As New EciB3()

            'Display program banner
            Console.WriteLine("CICS Transaction Gateway Basic ECI Sample 3")
            Console.WriteLine()

            'Display usage information
            Console.WriteLine("Usage: EciB3.exe [Gateway host name] [Gateway port number]")
            Console.WriteLine()

            If (eciApp.ParseArguments(args)) Then
                Try
                    eciApp.Start()
                Catch e As Exception
                    'Output exception details
                    Console.WriteLine(e.Message)
                    Console.WriteLine(e.StackTrace)
                End Try
            End If
            Console.ReadLine()
        End Sub

        ''' <summary>
        ''' Parses the command line arguments specified when the application was launched.
        ''' </summary>
        ''' <param name="args">An array of command line arguments.</param>
        ''' <returns><see langword="true"/> if the arguments are parsed successfully; otherwise <see langword="false"/>.</returns>
        Public Function ParseArguments(ByVal args As String()) As Boolean

            'Check argument count
            If (args.Length > 2) Then
                For i As Integer = 2 To args.Length - 1
                    Console.WriteLine("ERROR: Unexpected argument: {0}", args(i))
                Next
                Return False
            End If

            'Parse Gateway port number
            If (args.Length > 1) Then
                If (Not Integer.TryParse(args(1), gwyPort)) Then
                    Console.WriteLine("ERROR: Invalid port number specified")
                    Return False
                End If
            End If

            'Parse Gateway host name
            If (args.Length > 0) Then
                gwyHostName = args(0)
            End If

            Console.WriteLine("The address of the Gateway has been set to {0} port {1}", gwyHostName, gwyPort)
            Console.WriteLine()

            Return True
        End Function

        ''' <summary>
        ''' Connects to the CICS Transaction Gateway and flows an ECI request.
        ''' </summary>
        ''' <remarks>
        ''' The user is prompted to select a CICS server to execute the EC03 program on.
        ''' The contents of each Container in the returned Channel is displayed.
        ''' </remarks>
        Public Sub Start()

            Dim cicsServer As String
            Dim eciReq As EciRequest

            'Open the Gateway connection with an infinite timeout period
            'The 'Using' VB.NET construct ensures the GatewayConnection is closed
            'when it goes out of scope, even if an exception is thrown

            'A System.Net.SocketException will be thrown if the connection
            'attempt fails, and an IBM.CTG.GatewayException will be thrown if
            'an error occurs in the Gateway daemon
            Using gwyConnection As New GatewayConnection(gwyHostName, gwyPort)

                'Connected to the Gateway daemon

                'Get name of CICS server from the user
                cicsServer = SelectServer(gwyConnection)
                If (cicsServer Is Nothing) Then
                    'No servers defined, or user chose to quit
                    Return
                End If

                'Prepare the EciRequest
                eciReq = New EciRequest()
                eciReq.ServerName = cicsServer
                eciReq.Program = ProgramName
                eciReq.ExtendMode = EciExtendMode.EciNoExtend
                'Create the Channel
                Using channel As Channel = CreateChannel()
                    'Set the Channel on the ECIRequest
                    eciReq.Channel = channel

                    'Flow the request
                    If (FlowRequest(gwyConnection, eciReq)) Then

                        DisplayChannelContents(eciReq.Channel)

                    End If
                End Using
            End Using
        End Sub

        ''' <summary>
        ''' Creates a Channel with one CHAR Container to pass to the EC03 program
        ''' </summary>
        '''  <returns>A Channel object to be used for the ECI request</returns>
        Private Function CreateChannel() As Channel
            Console.WriteLine("Enter text to send to the CICS program (max 128 chars):")
            Dim text As String = Console.ReadLine()
            Dim channel As New Channel(CHANNEL_NAME)
            channel.CreateContainer(CONTAINER_NAME, text)
            Return channel
        End Function

        ''' <summary>
        ''' Prints a list of Containers in the Channel and the contents of each Container
        ''' </summary>
        Private Sub DisplayChannelContents(ByVal channel As Channel)
            Console.WriteLine("Program EC03 returned containers:")

            Dim containers As IList(Of Container)
            containers = channel.GetContainers()

            For Each aContainer As Container In containers
                If (aContainer.Type = ContainerType.BIT) Then
                    Console.Write("    [BIT] {0,-16} = ", aContainer.Name)
                    Console.WriteLine(BitConverter.ToString(aContainer.GetBitData()))
                Else
                    Console.Write("   [CHAR] {0,-16} = ", aContainer.Name)
                    Console.WriteLine(aContainer.GetCharData())
                End If
            Next aContainer
        End Sub

        ''' <summary>
        ''' Flows the ECI request, up to a maximum of MaxFlowAttempts attempts.
        ''' </summary>
        ''' <returns><see langword="true"/> if the flow succeeds; <see langword="false"/> if MaxFlowAttempts is reached.</returns>
        ''' <remarks>
        ''' If the flow returns a security error, the user is prompted to enter CICS logon credentials.
        ''' </remarks>
        Private Function FlowRequest(ByVal gwyConnection As GatewayConnection, ByVal eciReq As EciRequest) As Boolean

            Dim attempts As Integer = 0

            While (attempts < MaxFlowAttempts)

                'Flow throws a System.Net.SocketException if a connection
                'failure occurs, or an IBM.CTG.GatewayException if an error
                'occurs in the Gateway daemon
                gwyConnection.Flow(eciReq)

                'Test return code
                Select Case eciReq.EciReturnCode
                    Case EciReturnCode.EciNoError
                        'Flow succeeded
                        Return True

                    Case EciReturnCode.EciErrSecurityError
                        'Security error, prompt for CICS logon credentials
                        Console.Write("Enter CICS user id : ")
                        eciReq.UserId = Console.ReadLine()
                        Console.Write("Enter CICS password: ")
                        eciReq.Password = Console.ReadLine()

                    Case EciReturnCode.EciErrTransactionAbend
                        'Transaction abended
                        Console.WriteLine("The transaction abended: {0}", eciReq.AbendCode)
                        Return False

                    Case EciReturnCode.EciErrNoCics
                        'CICS server not found
                        Console.WriteLine("CICS server not found")
                        Return False

                    Case Else
                        'Other error return code
                        Console.WriteLine("ECI return code: {0} ({1})", eciReq.EciReturnCode.ToString(), CType(eciReq.EciReturnCode, Integer))
                        Return False

                End Select

                'Increment number of attempts
                attempts += 1
            End While

            'Number of attempts exceeded MaxFlowAttempts
            Return False
        End Function


        ''' <summary>
        ''' Prompts the user to select a server to execute the EC03 program on.
        ''' </summary>
        ''' <returns>The name of the CICS server, as defined in the CICS Transaction Gateway.</returns>
        Private Function SelectServer(ByVal gwyConnection As GatewayConnection) As String

            Dim servers As IList(Of CicsServer)
            Dim chosenServer As String = Nothing

            'Retrieve list of CICS servers from the Gateway

            'ListSystems throws a System.Net.SocketException if a connection
            'failure occurs, or an IBM.CTG.GatewayException if an error occurs
            'in the Gateway daemon
            servers = gwyConnection.ListSystems()

            If (servers.Count = 0) Then
                'There are no servers defined in the Gateway
                Console.WriteLine("No CICS servers have been defined in the Gateway")
                Return Nothing
            End If

            While (chosenServer Is Nothing)

                'Display a list of available CICS servers
                Console.WriteLine("CICS servers defined in the Gateway:")

                For i As Integer = 0 To servers.Count - 1
                    Console.WriteLine("   {0,2}. {1,-8} - {2}", (i + 1), servers(i).Name, servers(i).Description)
                Next
                Console.WriteLine()

                'Prompt for user selection
                Console.Write("Choose server to connect to, or Q to quit: ")
                chosenServer = Console.ReadLine()

                'Parse user selection
                If (chosenServer.Equals("Q", StringComparison.InvariantCultureIgnoreCase)) Then
                    'User chose to quit
                    Return Nothing
                End If

                Dim serverNum As Integer
                If (Integer.TryParse(chosenServer, serverNum)) Then

                    'Validate number
                    If ((serverNum < 1) Or (serverNum > servers.Count)) Then
                        Console.WriteLine("Invalid selection")
                        chosenServer = Nothing
                    Else
                        chosenServer = servers(serverNum - 1).Name
                    End If

                Else
                    'Input was not a number
                    Console.WriteLine("Invalid selection")
                    chosenServer = Nothing
                End If

                Console.WriteLine()
            End While

            Return chosenServer
        End Function

    End Class

End Namespace
